/*
 *                   jfjji,.           H A M M E R H E A D  S H A R K W A R E
 *                  ,#######EL:                  http://www.hhsw.de
 *                   .,j######Kj       
 *                    ;W########G:     This  code  is  free software; you can
 *                   j#####G ,D#L:     redistribute it and/or modify it under
 *             .,;;,G#####D.           the  terms  of  the GNU General Public
 *           jE##########G             License   as  published  by  the  Free
 *             .,W######D.             Software  Foundation; either version 2
 *             :L#####K##G             of  the  License,  or (at your option)
 *           .f#####G;.t##:            any later version.
 *         ;L###ELi.    ,D,            
 *       iEEGWj                        This  code  is distributed in the hope
 *      tW;                            that  it  will  be useful, but without
 *     jf                              any warranty; without even the implied
 *    .Kt                              warranty of merchantability or fitness
 *    i:                               for  a particular purpose. See the GNU
 *    ;                                General   Public   License   for  more
 *    .                                details (http://www.gnu.org).
 */
#ifndef bbdock_Slot_hh
#define bbdock_Slot_hh

#include <unistd.h>
#include <string>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>

#include <png.h>

#include "Dock.hh"
#include "Icon.hh"
#include "Render.hh"
#include "WindowManager.hh"

namespace bbdock
{
	/**
	 * Shows an icon in the BlackBox slit from wich the user
	 * can launch a corresponding application
	 *
	 * @author mf@hhsw.de
	 * @version 0.2.2
	 */
	class Slot
	{
		public:
			inline const Window &getWindow() const { return window; }
			inline Slot *getNext() const { return next; }
			inline const char *getIcon() const { 
				return icon->getImage().c_str(); }
			inline const char *getCommand() const { 
				return icon->getCommand().c_str(); }
			inline const Window getWID() { validateWID(); return wid; }
			inline const void setVisibility( int s ) { visibility = s; }
			inline const void invalidate() { invalid = true; }

			Slot( Dock &, Icon &, Slot * );
			virtual ~Slot();
			virtual void draw( bool = false );
			virtual void exec( int = 0, int = 0, char * = 0 );
			virtual void iconifyApplication();
			virtual void lowerApplication();
			virtual void closeApplication();
			virtual void tick();
			virtual void setIcon( const char * );

		protected:
			virtual void loadIcon();
			virtual void destroyIcon();
			virtual void createSurface();
			virtual void destorySurface();
			virtual void getBackground();
			virtual const bool validateWID();
			virtual const Window getSensitiveWindow( Window, long, int, int )
				const;

		private:
			enum
			{
				Complete = 255
			};

			Dock *dock;
			Icon *icon;
			Slot *next;
			Render *render;
			Window window;
			XImage *surface;
			int *normalicon;
			char *image;
			char *background;
			int left;
			int top;
			int width;
			int height;
			int visibility;
			bool invalid;
			WindowManager wm;
			int pid;
			Window wid;
			int fading;
			static const double fadestep;
	};
}

#endif
